mgMappingsDS = function() {
  this.init();
}

mgMappingsDS.prototype = {
  URI : "urn:mozgest:mappings-ui-datasource",
  prefix : "urn:mozgest-localized:",
  ds : null,
  source : null,
  loc : null,
  deletedNodes : new Array(),  // URN strings of all deleted nodes
  addedNodes : new Array(),    // URN strings of all added nodes

  init : function() {
    this.ds = Components.classes['@mozilla.org/rdf/datasource;1?name=in-memory-datasource']
              .createInstance(Components.interfaces.nsIRDFDataSource);

    // store a reference to our localizer (needed when window is closing)
    this.loc = mgMappingLocalizer;

    this.source = mgStorage.getUserRDF();

    this.copyUserStore();
  },
  
  copyUserStore : function() {
    var elements = this.source.GetAllResources();
    while (elements.hasMoreElements()) {
      var el = elements.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
      // only process Seqs
      if(mgStorage.RDFCUtils.IsSeq(this.source, el)) {
        var seqRes = el;
        var sourceSeq = mgStorage.newContainer();
        sourceSeq.Init(this.source, el);
        var targetSeq = mgStorage.newContainer();
        try {
          targetSeq.Init(this.ds, seqRes);
        } catch (e) {
          mgStorage.RDFCUtils.MakeSeq(this.ds, seqRes);
          targetSeq.Init(this.ds, seqRes);
        }

        var mappingsEnum = sourceSeq.GetElements();
        while (mappingsEnum.hasMoreElements()) {
          var mapping = mappingsEnum.getNext()
                        .QueryInterface(Components.interfaces.nsIRDFResource);
          this.loadMapping(mapping, targetSeq);
        }
      }
    }
  },

  // copies a mapping from userstore to memory datasource, while localizing
  // function names & codes. note that the mapping has the same URN as in the
  // userstore to make back-synchronization (see applyChanges) easier
  loadMapping : function(aMapping, aTargetSeq) {
    // get original values
    var oCode = mgStorage.getProperty(this.source, aMapping, "code");
    var oApp = mgStorage.getProperty(this.source, aMapping, "appearance");
    var oFunc = mgStorage.getProperty(this.source, aMapping, "function");
    var oCustom = mgStorage.getProperty(this.source, aMapping, "custom");
    var oName = mgStorage.getProperty(this.source, aMapping, "name");

    // localize movement code (excluding rockers)
    var locCode;
    if (oCode.substring(0,1) == ":") {
      locCode = oCode;
    } else {
      locCode = this.loc.localize(oCode);
    }

    // add mapping to memory datasource
    aTargetSeq.AppendElement(aMapping, true);
    mgStorage.assertLiteral(this.ds, aMapping, "code", locCode);
    if (oApp) {
      mgStorage.assertLiteral(this.ds, aMapping, "appearance",
                              this.loc.localize(oApp));
    }
    if (oFunc) {
      mgStorage.assertLiteral(this.ds, aMapping, "function", oFunc);
    }
    if (oCustom) {
      mgStorage.assertLiteral(this.ds, aMapping, "custom", oCustom);
    }
    mgStorage.assertLiteral(this.ds, aMapping, "name",
                            oName || this.loc.getString(oFunc));
  },

  mindAdded : function(aNode) {
    this.addedNodes[this.addedNodes.length] = aNode;
  },

  mindDeleted : function(aNode) {
    for (i = 0; i < this.addedNodes.length; i++) {
      if (this.addedNodes[i].Value == aNode.Value) {
        this.addedNodes[i] = null;
        return;
      }
    }
    this.deletedNodes[this.deletedNodes.length] = aNode;
  },

  applyChanges : function() {
    for (i = 0; i < this.addedNodes.length; i++) {
      this.saveMapping(this.addedNodes[i]);
    }
    for (i = 0; i < this.deletedNodes.length; i++) {
      this.deleteNode(this.deletedNodes[i]);
    }
  },

  // saves a mapping to the userstore, while deLocalizing it
  saveMapping : function(aMapping) {
    // get original values
    var code = mgStorage.getProperty(this.ds, aMapping, "code");
    var app = mgStorage.getProperty(this.ds, aMapping, "appearance");
    var func = mgStorage.getProperty(this.ds, aMapping, "function");
    var custom = mgStorage.getProperty(this.ds, aMapping, "custom");
    var name = mgStorage.getProperty(this.ds, aMapping, "name");

    // delocalize movement code (excluding rockers)
    var stdCode;
    if (code.substring(0,1) == ":") {
      stdCode = code;
    } else {
      stdCode = this.loc.deLocalize(code);
    }

    // determine the mapping's parent container
    var container = null;
    var arcs = this.ds.ArcLabelsIn(aMapping);
    while(arcs.hasMoreElements()) {
      var arc = arcs.getNext();
      var parent = this.ds.GetSource(arc, aMapping, true);
      if (mgStorage.RDFCUtils.IsContainer(this.ds, parent)) {
        var container = mgStorage.newContainer();
        try {
          container.Init(this.source, parent);
        } catch (e) {
          alert("MozGest fatal error (mappingsDS.saveMapping):\n" + e + "\n");
          return;
        }
      }
    }
    // save mapping to userstore
    container.AppendElement(aMapping, true);
    mgStorage.assertLiteral(this.source, aMapping, "code", stdCode);
    if (app) {
      mgStorage.assertLiteral(this.source, aMapping, "appearance",
                              this.loc.deLocalize(app));
    }
    if (func) {
      mgStorage.assertLiteral(this.source, aMapping, "function", func);
    }
    if (custom) {
      mgStorage.assertLiteral(this.source, aMapping, "custom", custom);
      mgStorage.assertLiteral(this.source, aMapping, "name", name);
    }
  },

  // Deletes a node from the userstore
  deleteNode : function(aNode) {
    if (!aNode) {
      return;
    }
    // WARNING: this is intended only for nodes inside a container. behaviour
    // with nodes that have multiple or non-container inward-arcs is untested!
    var arcs = this.source.ArcLabelsIn(aNode);
    while(arcs.hasMoreElements()) {
      var arc = arcs.getNext();
      var parent = this.source.GetSource(arc, aNode, true);
      if (mgStorage.RDFCUtils.IsContainer(this.source, parent)) {
        var container = mgStorage.newContainer();
        container.Init(this.source, parent);
        container.RemoveElement(aNode, true);
      }
    }
    var arcs = this.source.ArcLabelsOut(aNode);
    while(arcs.hasMoreElements()) {
      var arc = arcs.getNext();
      var targets = this.source.GetTargets(aNode, arc, true);
      while (targets.hasMoreElements()) {
        var target = targets.getNext();
        this.source.Unassert(aNode, arc, target, true);
      }
    }
  },


  // nsIRDFDatasource wrapper implementation

  GetSource : function(aProperty, aTarget, aTruthValue) {
    return this.ds.GetSource(aProperty, aTarget, aTruthValue);
  },
  GetSources : function(aProperty, aTarget, aTruthValue) {
    return this.ds.GetSources(aProperty, aTarget, aTruthValue);
  },
  GetTarget : function(aSource, aProperty, aTruthValue) {
    return this.ds.GetTarget(aSource, aProperty, aTruthValue);
  },
  GetTargets: function(aSource, aProperty, aTruthValue) {
    return this.ds.GetTargets(aSource, aProperty, aTruthValue);
  },
  HasAssertion: function(aSource, aProperty, aTarget, aTruthValue) {
    return this.ds.HasAssertion(aSource, aProperty, aTarget, aTruthValue);
  },
  AddObserver: function(aObserver) {
    return this.ds.AddObserver(aObserver);
  },
  RemoveObserver: function(aObserver) {
    return this.ds.RemoveObserver(aObserver);
  },
  ArcLabelsIn: function(aNode) {
    return this.ds.ArcLabelsIn(aNode);
  },
  hasArcOut: function(aSource, aArc) {
    return this.ds.hasArcOut(aSource, aArc);
  },

  Assert: function(aSource, aProperty, aTarget, aTruthValue) {
    return this.ds.Assert(aSource, aProperty, aTarget, aTruthValue);
  },
  Unassert: function(aSource, aProperty, aTarget) {
    return this.ds.Unassert(aSource, aProperty, aTarget);
  },
  Change: function(aSource, aProperty, aOldTarget, aNewTarget) {
    return this.ds.Change(aSource, aProperty, aOldTarget, aNewTarget);
  },
  Move: function(aOldSource, aNewSource, aProperty, aTarget) {
    return this.ds.Move(aOldSource, aNewSource, aProperty, aTarget);
  },

  ArcLabelsOut: function(aSource) {
    return this.ds.ArcLabelsOut(aSource);
  },
  GetAllResources : function() {
    return this.ds.GetAllResources();
  },
  IsCommandEnabled: function(aSources, aCommand, aArguments) {
    return this.ds.IsCommandEnabled(aSources, aCommand, aArguments);
  },
  DoCommand: function(aSources, aCommand, aArguments) {
    return this.ds.DoCommand(aSources, aCommand, aArguments);
  },
  GetAllCmds: function(aSource) {
    return this.ds.GetAllCmds(aSource);
  },
  hasArcIn: function(aNode, aArc) {
    return this.ds.hasArcIn(aNode, aArc);
  },

  beginUpdateBatch: function() {
  },
  endUpdateBatch: function() {
  },
}

