// Session Saver 0.2
// By Pike

var SessionSaver = {

	prefService: null,
	prefBranch: 'sessionsaver.',
	defaultName: 'default_session',
	appendInterval: 10000, // In milliseconds
	
	onLoad: function()
	{
		// Setup object properties
		this.prefService = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch(null);
		try
		{
			this.appendInterval = this.prefService.getIntPref(this.prefBranch + 'settings.append_interval');
		}
		catch (err) {}
		if (this.appendInterval < 0) this.appendInterval = 0;
		
		// Restore session in first opened window
		if (this.countBrowserWindows() == 1)
		{
			var autoLoad = true;
			try
			{
				autoLoad = this.prefService.getBoolPref(this.prefBranch + 'settings.autoload');
			}
			catch (err) {}
			if (autoLoad)
			{
				// Find user's home page(s)
				var homepages;
				try
				{
					homepages = this.prefService.getComplexValue('browser.startup.homepage', Components.interfaces.nsIPrefLocalizedString).data;
				}
				catch (err) {}
				
				// Check for window URL arguments
				var overwriteTabs = true;
				if ('arguments' in window && window.arguments.length >= 1 && window.arguments[0])
				{
					// Overwrite existing tabs unless a non-homepage URL argument was passed
					for (var i = 0; i < window.arguments.length; i++)
					{
						if (homepages.indexOf(window.arguments[i]) == -1)
						{
							overwriteTabs = false;
							break;
						}
					}
				}
				
				// Load saved session
				this.loadSession(null, overwriteTabs);
			}
		}
	},
	
	onUnload: function()
	{
		// Check if autosave is enabled
		var autoSave = true;
		try
		{
			autoSave = this.prefService.getBoolPref(this.prefBranch + 'settings.autosave');
		}
		catch (err) {}
		if (autoSave)
		{
			// Only allow session appends to occur within the current browser session
			var allowNextSessionToAppend = (this.countBrowserWindows() > 0 ? true : false);
			this.saveSession(null, allowNextSessionToAppend);
		}
	},
	
	saveSession: function(saveName, storeCloseTime)
	{
		if (!saveName) saveName = this.defaultName;
		const selectedPrefName = this.prefBranch + saveName + '.selected';
		const lastClosePrefName = this.prefBranch + saveName + '.last_close';
		
		// Calculate whether to append to existing session based on time since last window close
		var appendSession = false;
		try
		{
			var currentTime = new Date().valueOf();
			var lastChecked = this.prefService.getCharPref(lastClosePrefName);
			if (currentTime - lastChecked <= this.appendInterval) appendSession = true;
		}
		catch (err) {}
		
		// Prepare new session by finding end of last session or clearing last session
		var tabCount = 0;
		if (appendSession)
		{
			// Get number of stored tabs from prefs
			var prefChildList = this.prefService.getChildList(this.prefBranch + saveName + '.', {});
			for (var i = 0; i < prefChildList.length; i++)
			{
				if (prefChildList[i] != selectedPrefName && prefChildList[i] != lastClosePrefName)
				{
					tabCount++;
				}
			}
		}
		else
		{
			this.prefService.deleteBranch(this.prefBranch + saveName + '.');
		}
		
		// Save history locations
		var selectedTab = 0;
		for (var i = 0; i < getBrowser().browsers.length; i++)
		{
			try
			{
				var oldHistory = getBrowser().browsers[i].sessionHistory;
			}
			catch (err)
			{
				// If no session history available then assume empty and skip to next tab
				continue;
			}
			var savedHistory = oldHistory.index;
			if (savedHistory > -1)
			{
				for (var j = 0; j < oldHistory.count; j++)
				{
					savedHistory += '|';
					savedHistory += oldHistory.getEntryAtIndex(j, false).URI.spec;
				}
				this.prefService.setCharPref(this.prefBranch + saveName + '.' + tabCount, savedHistory);
				if (getBrowser().browsers[i] == getBrowser().mCurrentBrowser) selectedTab = tabCount;
				tabCount++;
			}
		}
		
		// If any tabs stored then save selected tab position
		if (tabCount)
		{
			this.prefService.setIntPref(selectedPrefName, selectedTab);
		}
		
		// Store or remove last_close pref as necessary
		if (storeCloseTime)
		{
			this.prefService.setCharPref(lastClosePrefName, new Date().valueOf());
		}
		else if (this.prefService.getPrefType(lastClosePrefName))
		{
			this.prefService.clearUserPref(lastClosePrefName);
		}
	},
	
	loadSession: function(loadName, overwriteTabs)
	{
		if (!loadName) loadName = this.defaultName;
		
		// Get list of stored tabs from prefs
		var prefChildList = this.prefService.getChildList(this.prefBranch + loadName + '.', {});
		var storedTabList = new Array();
		var selectedTab = 0;
		for (var i = 0; i < prefChildList.length; i++)
		{
			if (prefChildList[i] == this.prefBranch + loadName + '.selected')
			{
				selectedTab = this.prefService.getIntPref(prefChildList[i]);
			}
			else if (prefChildList[i] != this.prefBranch + loadName + '.last_close')
			{
				storedTabList.push(prefChildList[i]);
			}
		}
		
		// Open necessary tabs
		var currentTabList = new Array();
		var openTabCount = getBrowser().browsers.length;
		for (var tab = 0; tab < storedTabList.length; tab++)
		{
			if (!overwriteTabs || tab >= openTabCount)
			{
				// Open new tab
				currentTabList.push(getBrowser().addTab());
			}
			else if (overwriteTabs && tab < openTabCount)
			{
				// Reuse existing tab
				currentTabList.push(getBrowser().mTabContainer.childNodes[tab]);
			}
		}
		
		// Close unwanted tabs
		if (overwriteTabs)
		{
			for (var i = storedTabList.length; i < openTabCount; i++)
			{
				getBrowser().removeTab(i);
			}
		}
		
		// Restore history
		window.setTimeout(function() {SessionSaver.restoreHistory(loadName, storedTabList, currentTabList, selectedTab, overwriteTabs);}, 100);
	},
	
	restoreHistory: function(loadName, storedTabList, currentTabList, selectedTab, overwriteTabs)
	{
		var ioService = Components.classes['@mozilla.org/network/io-service;1'].getService(Components.interfaces.nsIIOService);
		for (var tab = 0; tab < storedTabList.length; tab++)
		{
			var curWebNav = getBrowser().getBrowserForTab(currentTabList[tab]).webNavigation;
			
			// Retrieve session history of current tab
			var newHistory = curWebNav.sessionHistory;
			newHistory.QueryInterface(Components.interfaces.nsISHistoryInternal);
			
			// If necessary clear session history
			if (newHistory.count > 0) newHistory.PurgeHistory(newHistory.count);
			
			// Clone history from pref
			var savedHistory = this.prefService.getCharPref(this.prefBranch + loadName + '.' + tab).split('|');
			for (var i = 1; i < savedHistory.length; i++)
			{
				var newEntry = Components.classes['@mozilla.org/browser/session-history-entry;1'].createInstance(Components.interfaces.nsISHEntry);
				var newURI = ioService.newURI(savedHistory[i], null, null);
				newEntry.SetURI(newURI);
				newHistory.addEntry(newEntry, true);
			}
			
			// Load stored history location
			if (savedHistory.length > 1 && savedHistory[0] >= 0)
			{
				curWebNav.gotoIndex(savedHistory[0]);
			}
			
			// Reselect tab that was open when session was stored
			if (tab == selectedTab && overwriteTabs)
			{
				getBrowser().selectedTab = currentTabList[tab];
			}
		}
	},
	
	countBrowserWindows: function()
	// Returns number of open browser windows (ignores bookmark manager, javascript console, etc.)
	{
		var windowMediator = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
		var enumerator = windowMediator.getEnumerator('navigator:browser');
		var numWindows = 0;
		while (enumerator.hasMoreElements())
		{
			var foundWindow = enumerator.getNext();
			if (foundWindow._content) numWindows++;
		}
		return numWindows;
	}
};

window.addEventListener('load', function() {SessionSaver.onLoad();}, false);
window.addEventListener('unload', function() {SessionSaver.onUnload();}, false);
